@@ -52,7 +52,10 @@ from tests.common.debug import (
     find_any,
     minimal,
 )
-from tests.common.utils import Why, fails_with, temp_registered, xfail_on_crosshair
+from tests.common.utils import fails_with, temp_registered
+
+# we'll continue testing the typing variants until their removal from the stdlib
+# ruff: noqa: UP006, UP035
 
 sentinel = object()
 BUILTIN_TYPES = tuple(
@@ -619,7 +622,10 @@ class Tree:
         return f"Tree({self.left}, {self.right})"
 
 
-@pytest.mark.skipif(settings._current_profile == "crosshair", reason="takes ~19 mins")
+@pytest.mark.skipif(
+    settings._current_profile == "crosshair",
+    reason="takes ~19 mins; datastructure explosion https://github.com/pschanely/hypothesis-crosshair/issues/27",
+)
 @given(tree=st.builds(Tree))
 def test_resolving_recursive_type(tree):
     assert isinstance(tree, Tree)
@@ -854,7 +860,6 @@ def test_bytestring_not_treated_as_generic_sequence(val):
         assert isinstance(x, set)
 
 
-@pytest.mark.skipif(sys.version_info[:2] >= (3, 14), reason="FIXME-py314")
 @pytest.mark.parametrize(
     "type_", [int, Real, object, typing.Union[int, str], typing.Union[Real, str]]
 )
@@ -876,7 +881,6 @@ def test_supportsop_types_support_protocol(protocol, data):
     assert issubclass(type(value), protocol)
 
 
-@xfail_on_crosshair(Why.undiscovered)
 @pytest.mark.parametrize("restrict_custom_strategy", [True, False])
 def test_generic_aliases_can_be_conditionally_resolved_by_registered_function(
     restrict_custom_strategy,
@@ -983,7 +987,7 @@ def test_no_byteswarning(_):
 
 @pytest.mark.skipif(
     settings._current_profile == "crosshair",
-    reason="Crosshair doesn't generate the Decimal('snan'), so this runs for hours",
+    reason="Crosshair is too much slower at hashing values",
 )
 def test_hashable_type_unhashable_value():
     # Decimal("snan") is not hashable; we should be able to generate it.
@@ -1147,7 +1151,17 @@ def test_resolves_builtin_types(t):
 
 @pytest.mark.parametrize("t", BUILTIN_TYPES, ids=lambda t: t.__name__)
 @given(data=st.data())
+@settings(max_examples=20)
 def test_resolves_forwardrefs_to_builtin_types(t, data):
+    if t.__name__ == "object" and settings._current_profile == "threading":
+        # from_type(ForwardRef("object")) pulls from register_type_strategy,
+        # and depending on threading I've seen `st.builds(Bar, st.integers())`
+        # (from this file) be registered in one iteration and not the next,
+        # causing Hypothesis to raise FlakyStrategyDefinition.
+        #
+        # (I would also expect st.from_type(object) to have this problem, but
+        # I haven't seen that error under threading, yet).
+        pytest.skip("ForwardRef('object') is inherently flaky under concurrency")
     s = st.from_type(typing.ForwardRef(t.__name__))
     v = data.draw(s)
     assert isinstance(v, t)
@@ -1167,15 +1181,24 @@ def test_resolves_type_of_union_of_forwardrefs_to_builtins(x):
 
 
 @pytest.mark.parametrize(
-    # Old-style `List` because `list[int]() == list()`, so no need for the hint.
     "type_",
-    [getattr(typing, "List", None)[int], typing.Optional[int]],
+    [
+        # Old-style `List` because `list[int]() == list()`, so no need for the hint.
+        getattr(typing, "List", None)[int],
+        pytest.param(
+            typing.Optional[int],
+            marks=pytest.mark.skipif(
+                sys.version_info >= (3, 14), reason="different error on 3.14+"
+            ),
+        ),
+    ],
 )
 def test_builds_suggests_from_type(type_):
     with pytest.raises(
         InvalidArgument, match=re.escape(f"try using from_type({type_!r})")
     ):
         check_can_generate_examples(st.builds(type_))
+
     try:
         check_can_generate_examples(st.builds(type_, st.just("has an argument")))
         raise AssertionError("Expected strategy to raise an error")
@@ -1183,6 +1206,20 @@ def test_builds_suggests_from_type(type_):
         assert not isinstance(err, InvalidArgument)
 
 
+@pytest.mark.skipif(sys.version_info < (3, 14), reason="different error on 3.14+")
+@pytest.mark.parametrize("type_", [typing.Optional[int]])
+def test_builds_suggests_from_type_on_construction(type_):
+    with pytest.raises(
+        InvalidArgument, match=re.escape(f"Try using from_type({type_!r})")
+    ):
+        check_can_generate_examples(st.builds(type_))
+
+    with pytest.raises(
+        InvalidArgument, match=re.escape(f"Try using from_type({type_!r})")
+    ):
+        check_can_generate_examples(st.builds(type_, st.just("has an argument")))
+
+
 def test_builds_mentions_no_type_check():
     @typing.no_type_check
     def f(x: int):
@@ -1256,8 +1293,9 @@ def test_issue_4194_regression():
     inner = typing.Union[typing.Sequence["A"], MyProtocol]
     A = typing.Union[typing.Sequence[inner], MyProtocol]
 
-    with (
-        temp_registered(MyProtocol, st.just(b"")),
-        temp_registered(typing.ForwardRef("A"), st.integers()),
-    ):
-        find_any(st.from_type(A))
+    with temp_registered(MyProtocol, st.just(b"")):
+        # NOTE: For compatibility with Python 3.9's LL(1)
+        # parser, this is written as a nested with-statement,
+        # instead of a compound one.
+        with temp_registered(typing.ForwardRef("A"), st.integers()):
+            find_any(st.from_type(A))
