@@ -126,7 +126,10 @@ class Node:
     right: typing.Union["Node", int]
 
 
-@pytest.mark.skipif(settings._current_profile == "crosshair", reason="takes ~11 mins")
+@pytest.mark.skipif(
+    settings._current_profile == "crosshair",
+    reason="takes ~11 mins; datastructure explosion: https://github.com/pschanely/hypothesis-crosshair/issues/27",
+)
 @given(st.builds(Node))
 def test_can_resolve_recursive_dataclass(val):
     assert isinstance(val, Node)
@@ -197,6 +200,14 @@ def has_posonly_args(x, /, y):
     pass
 
 
+@pytest.mark.xfail(
+    settings._current_profile == "threading",
+    reason=(
+        "dynamic @example applications modify the shared "
+        "has_posonly_args.hypothesis._given_kwargs."
+    ),
+    strict=False,
+)
 def test_example_argument_validation():
     example(y=None)(has_posonly_args)(1)  # Basic case is OK
 
