@@ -12,6 +12,8 @@ import datetime
 import os
 import subprocess
 import sys
+from contextlib import contextmanager
+from threading import RLock
 from unittest import TestCase
 
 import pytest
@@ -26,11 +28,10 @@ from hypothesis._settings import (
     note_deprecation,
     settings,
 )
-from hypothesis.database import ExampleDatabase, InMemoryExampleDatabase
+from hypothesis.database import InMemoryExampleDatabase
 from hypothesis.errors import (
     HypothesisDeprecationWarning,
     InvalidArgument,
-    InvalidState,
 )
 from hypothesis.stateful import RuleBasedStateMachine, rule
 from hypothesis.utils.conventions import not_set
@@ -40,21 +41,38 @@ from tests.common.utils import (
     counts_calls,
     fails_with,
     skipif_emscripten,
+    skipif_threading,
     validate_deprecation,
 )
 
-
-def test_has_docstrings():
-    assert settings.verbosity.__doc__
-
-
 original_default = settings.get_profile("default").max_examples
+_temp_register_profile_lock = RLock()
 
 
-def setup_function(fn):
-    settings.load_profile("default")
-    settings.register_profile("test_settings", settings())
-    settings.load_profile("test_settings")
+@contextmanager
+def temp_register_profile(name, parent, **kwargs):
+    with _temp_register_profile_lock:
+        try:
+            settings.register_profile(name, parent, **kwargs)
+            yield
+        finally:
+            settings._profiles.pop(name)
+
+
+_restore_profile_lock = RLock()
+
+
+@contextmanager
+def restore_profile():
+    with _restore_profile_lock:
+        # avoid polluting global state by resetting the loaded profile back to its
+        # previous value in tests which use load_profile
+        current_profile = settings._current_profile
+        assert current_profile is not None
+        try:
+            yield
+        finally:
+            settings.load_profile(current_profile)
 
 
 def test_cannot_set_non_settings():
@@ -80,7 +98,7 @@ def test_respects_none_database():
 def test_can_repeatedly_push_the_same_thing():
     s = settings(max_examples=12)
     t = settings(max_examples=17)
-    assert settings().max_examples == original_default
+    original = settings().max_examples
     with local_settings(s):
         assert settings().max_examples == 12
         with local_settings(t):
@@ -92,7 +110,7 @@ def test_can_repeatedly_push_the_same_thing():
                 assert settings().max_examples == 12
             assert settings().max_examples == 17
         assert settings().max_examples == 12
-    assert settings().max_examples == original_default
+    assert settings().max_examples == original
 
 
 def test_can_set_verbosity():
@@ -107,7 +125,7 @@ def test_can_not_set_verbosity_to_non_verbosity():
         settings(verbosity="kittens")
 
 
-@pytest.mark.parametrize("db", [None, ExampleDatabase()])
+@pytest.mark.parametrize("db", [None, InMemoryExampleDatabase()])
 def test_inherits_an_empty_database(db):
     with local_settings(settings(database=InMemoryExampleDatabase())):
         assert settings.default.database is not None
@@ -118,7 +136,7 @@ def test_inherits_an_empty_database(db):
         assert t.database is db
 
 
-@pytest.mark.parametrize("db", [None, ExampleDatabase()])
+@pytest.mark.parametrize("db", [None, InMemoryExampleDatabase()])
 def test_can_assign_database(db):
     x = settings(database=db)
     assert x.database is db
@@ -131,20 +149,23 @@ def test_will_reload_profile_when_default_is_absent():
 
 
 def test_load_profile():
-    settings.load_profile("default")
-    assert settings.default.max_examples == original_default
-    assert settings.default.stateful_step_count == 50
+    with restore_profile():
+        settings.load_profile("default")
+        assert settings.default.max_examples == original_default
+        assert settings.default.stateful_step_count == 50
 
-    settings.register_profile("test", settings(max_examples=10), stateful_step_count=5)
-    settings.load_profile("test")
+        settings.register_profile(
+            "test", settings(max_examples=10), stateful_step_count=5
+        )
+        settings.load_profile("test")
 
-    assert settings.default.max_examples == 10
-    assert settings.default.stateful_step_count == 5
+        assert settings.default.max_examples == 10
+        assert settings.default.stateful_step_count == 5
 
-    settings.load_profile("default")
+        settings.load_profile("default")
 
-    assert settings.default.max_examples == original_default
-    assert settings.default.stateful_step_count == 50
+        assert settings.default.max_examples == original_default
+        assert settings.default.stateful_step_count == 50
 
 
 def test_profile_names_must_be_strings():
@@ -157,12 +178,13 @@ def test_profile_names_must_be_strings():
 
 
 def test_loading_profile_keeps_expected_behaviour():
-    settings.register_profile("ci", settings(max_examples=10000))
-    settings.load_profile("ci")
-    assert settings().max_examples == 10000
-    with local_settings(settings(max_examples=5)):
-        assert settings().max_examples == 5
-    assert settings().max_examples == 10000
+    with restore_profile():
+        settings.register_profile("ci", settings(max_examples=10000))
+        settings.load_profile("ci")
+        assert settings().max_examples == 10000
+        with local_settings(settings(max_examples=5)):
+            assert settings().max_examples == 5
+        assert settings().max_examples == 10000
 
 
 def test_load_non_existent_profile():
@@ -170,17 +192,6 @@ def test_load_non_existent_profile():
         settings.get_profile("nonsense")
 
 
-def test_cannot_delete_a_setting():
-    x = settings()
-    with pytest.raises(AttributeError):
-        del x.max_examples
-    x.max_examples
-
-    x = settings()
-    with pytest.raises(AttributeError):
-        del x.foo
-
-
 def test_cannot_set_settings():
     x = settings()
     with pytest.raises(AttributeError):
@@ -195,7 +206,7 @@ def test_can_have_none_database():
     assert settings(database=None).database is None
 
 
-@pytest.mark.parametrize("db", [None, ExampleDatabase(":memory:")])
+@pytest.mark.parametrize("db", [None, InMemoryExampleDatabase()])
 @pytest.mark.parametrize("bad_db", [":memory:", ".hypothesis/examples"])
 def test_database_type_must_be_ExampleDatabase(db, bad_db):
     with local_settings(settings(database=db)):
@@ -205,11 +216,6 @@ def test_database_type_must_be_ExampleDatabase(db, bad_db):
         assert settings.database is settings_property_db
 
 
-def test_cannot_define_settings_once_locked():
-    with pytest.raises(InvalidState):
-        settings._define_setting("hi", "there", default=4)
-
-
 def test_cannot_assign_default():
     with pytest.raises(AttributeError):
         settings.default = settings(max_examples=3)
@@ -232,6 +238,9 @@ def test_settings_alone():
 """
 
 
+# runpytest_inprocess uses invalidate_caches in pytest, which is not thread safe
+# (I presume; produces keyerrors).
+@skipif_threading
 def test_settings_alone(pytester):
     # Disable cacheprovider, since we don't need it and it's flaky on pyodide
     script = pytester.makepyfile(TEST_SETTINGS_ALONE)
@@ -326,6 +335,8 @@ class TestGivenExampleSettingsExplicitCalled(TestCase):
     @given(st.booleans())
     @example(True)
     @settings(phases=[Phase.explicit])
+    # counts_calls is not thread safe (modifying global f.calls attr)
+    @skipif_threading
     def test_example_explicit(self, x):
         self.call_target()
 
@@ -444,7 +455,7 @@ def test_assigning_to_settings_attribute_on_state_machine_raises_error():
 
 def test_derandomise_with_explicit_database_is_invalid():
     with pytest.raises(InvalidArgument):
-        settings(derandomize=True, database=ExampleDatabase(":memory:"))
+        settings(derandomize=True, database=InMemoryExampleDatabase())
 
 
 @pytest.mark.parametrize(
@@ -560,9 +571,9 @@ def test_check_defaults_to_randomize_when_not_running_on_ci():
     )
 
 
+@skipif_threading  # modifying global state (profiles) during testing
 def test_reloads_the_loaded_profile_if_registered_again():
-    prev_profile = settings._current_profile
-    try:
+    with restore_profile():
         test_profile = "some nonsense profile purely for this test"
         test_value = 123456
         settings.register_profile(test_profile, settings(max_examples=test_value))
@@ -571,9 +582,6 @@ def test_reloads_the_loaded_profile_if_registered_again():
         test_value_2 = 42
         settings.register_profile(test_profile, settings(max_examples=test_value_2))
         assert settings.default.max_examples == test_value_2
-    finally:
-        if prev_profile is not None:
-            settings.load_profile(prev_profile)
 
 
 CI_TESTING_SCRIPT = """
@@ -595,3 +603,10 @@ def test_will_automatically_pick_up_changes_to_ci_profile_in_ci():
         text=True,
         encoding="utf-8",
     )
+
+
+def test_register_profile_avoids_intermediate_profiles():
+    parent = settings()
+    s = settings(parent, max_examples=10)
+    with temp_register_profile("for_intermediate_test", s):
+        assert settings.get_profile("for_intermediate_test")._fallback is parent
