@@ -33,7 +33,16 @@ from tests.common.debug import (
     check_can_generate_examples,
     find_any,
 )
-from tests.common.utils import Why, fails_with, temp_registered, xfail_on_crosshair
+from tests.common.utils import (
+    Why,
+    fails_with,
+    skipif_threading,
+    temp_registered,
+    xfail_on_crosshair,
+)
+
+# we'll continue testing the typing variants until their removal from the stdlib
+# ruff: noqa: UP006, UP035
 
 types_with_core_strat = {
     type_
@@ -42,7 +51,6 @@ types_with_core_strat = {
 }
 
 
-@pytest.mark.skipif(sys.version_info[:2] >= (3, 14), reason="FIXME-py314")
 def test_generic_sequence_of_integers_may_be_lists_or_bytes():
     strat = st.from_type(Sequence[int])
     find_any(strat, lambda x: isinstance(x, bytes))
@@ -161,6 +169,9 @@ def test_custom_type_resolution_with_function():
 
 def test_custom_type_resolution_with_function_non_strategy():
     with temp_registered(UnknownType, lambda _: None):
+        # NOTE: For compatibility with Python 3.9's LL(1)
+        # parser, this is written as a nested with-statement,
+        # instead of a compound one.
         with pytest.raises(ResolutionFailed):
             check_can_generate_examples(st.from_type(UnknownType))
         with pytest.raises(ResolutionFailed):
@@ -195,6 +206,7 @@ def test_errors_if_generic_resolves_empty():
             check_can_generate_examples(fails_2)
 
 
+@skipif_threading
 def test_cannot_register_empty():
     # Cannot register and did not register
     with pytest.raises(InvalidArgument):
